package ru.ifmo.vizi.ukkonen.SuffixTree;

import java.lang.Math;
import ru.ifmo.vizi.base.widgets.Shape;
import ru.ifmo.vizi.base.widgets.ShapeStyle;

import java.awt.*;

/**
 * Suffix links class.
 *
 * @author  Igor Ahmetov
 * @version $Id$
 */
public class SuffixLink extends Shape {
    static public double arcHeight = 0.15;

    /**
     * Arrow height.
     */
    static public double arrowHeight = 10;

    /**
     * Arrow width.
     */
    static public double arrowWidth = 4;

    /**
     * Beginning of the suffix link.
     */
    protected NodeShape from;

    /**
     * End of the suffix link.
     */
    protected NodeShape to;

    /**
     * Default constructor.
     *
     * @param styleSet style set.
     * @param from beginning of the suffix link.
     * @param to end of the suffix link.
     */
    public SuffixLink(ShapeStyle[] styleSet, NodeShape from, NodeShape to) {
        super(styleSet);

        this.from = from;
        this.to = to;
    }

    /**
     * Returns square of the argument.
     *
     * @param x argument.
     * @return square of <code>x</code>.
     */
    protected double sqr(double x) {
        return x * x;
    }

    /**
     * Returns source node of this suffix link.
     *
     * @return source node of this suffix link.
     */
    public NodeShape getSourceNodeShape() {
        return from;
    }

    /**
     * Returns target node of this suffix link.
     *
     * @return target node of this suffix link.
     */
    public NodeShape getTargetNodeShape() {
        return to;
    }

    protected double getAngle(double x, double y) {
        double a = (x == 0) ? (y >= 0 ? Math.PI / 2 : Math.PI * 3 / 2) :
                Math.atan(y / x);
        if (x < 0) a += Math.PI;
        return a;
    }

    protected Dimension fit(Dimension size) {
        Rectangle fromBounds = from.getBounds();
        Rectangle toBounds = to.getBounds();

        int x1 = fromBounds.x + fromBounds.width / 2;
        int y1 = fromBounds.y + fromBounds.height / 2;
        int x2 = toBounds.x + toBounds.width / 2;
        int y2 = toBounds.y + toBounds.height / 2;

        return new Dimension(Math.abs(x1 - x2), Math.abs(y1 - y2));
    }

    public void paint(Graphics g) {
        Rectangle fromBounds = from.getBounds();
        Rectangle toBounds = to.getBounds();
        int x1 = fromBounds.x + fromBounds.width / 2;
        int y1 = fromBounds.y + fromBounds.height / 2;
        int x2 = toBounds.x + toBounds.width / 2;
        int y2 = toBounds.y + toBounds.height / 2;

        double a = y1 - y2, b = x2 - x1;
        double angle = getAngle(a, b);
        double l = Math.sqrt((x1 - x2) * (x1 - x2) + (y1 - y2) * (y1 - y2));
        double arc = arcHeight * l;
        double r = (4 * sqr(arc) + sqr(l)) / (8 * arc);
        double xPos = (x1 + x2) / 2 + (r - arc) * Math.cos(angle);
        double yPos = (y1 + y2) / 2 + (r - arc) * Math.sin(angle);
        double startAngle = getAngle(x1 - xPos, y1 - yPos) / Math.PI * 180;
        double endAngle = getAngle(x2 - xPos, y2 - yPos) / Math.PI * 180;
        if (endAngle < startAngle) endAngle += 360;

        if (look.getBorderStatus(style)) {
            g.setColor(look.getBorderColor(style));
            g.drawArc((int) Math.round(xPos - r), (int) Math.round(yPos - r),
                    (int) Math.round(2 * r), (int) Math.round(2 * r),
                    -(int) Math.round(startAngle), -(int) Math.round((endAngle - startAngle)));
        }

        angle = getAngle(x2 - xPos, y2 - yPos) - Math.PI / 2;

        r = toBounds.width / 2;

        int x[] = new int[3], y[] = new int[3];
        x[0] = x2 + (int) Math.round(r * Math.cos(angle));
        y[0] = y2 + (int) Math.round(r * Math.sin(angle));
        x[1] = x[0] + (int) Math.round(arrowHeight * Math.cos(angle) + arrowWidth * Math.sin(angle));
        y[1] = y[0] + (int) Math.round(arrowHeight * Math.sin(angle) - arrowWidth * Math.cos(angle));
        x[2] = x[0] + (int) Math.round(arrowHeight * Math.cos(angle) - arrowWidth * Math.sin(angle));
        y[2] = y[0] + (int) Math.round(arrowHeight * Math.sin(angle) + arrowWidth * Math.cos(angle));

        if (look.getFillStatus(style)) {
            g.setColor(look.getFillColor(style));
            g.fillPolygon(x, y, 3);
        }

        if (look.getBorderStatus(style)) {
            g.setColor(look.getBorderColor(style));
            g.drawPolygon(x, y, 3);
        }
    }
}
