package ru.ifmo.vizi.ukkonen.SuffixTree;

import java.util.Vector;

/**
 * Stores suffix tree.
 *
 * @author	Igor Ahmetov
 * @version	$Id$
 */
public class SuffixTreeData {
    /**
     * Debug flag.
     */
    public final static boolean DEBUG = true;

	/**
	 * String data for this tree.
	 */
	protected String string;

	/**
	 * Root node.
	 */
	protected SuffixTreeNode root;

    /**
     * States of the algorithm.
     */
    protected Vector states;

    /**
     * Current step.
     */
    protected int step;

    /**
     * Current extension.
     */
    protected int extension;

    /**
     * Current phase;
     */
    protected int phase;

	/**
	 * Default constructor.
	 *
	 * @param string	string for suffix tree.
	 */
	public SuffixTreeData(String string) {
		this.string = string;
        states = new Vector();

        buildTree();
	}

    /**
     * Builds tree for given string.
     */
    private void buildTree() {
        if (Options.DEBUG) System.out.println("SuffixTreeData.buildTree()");

        SuffixTreeState state;
        states.addElement(
                new SuffixTreeState(-2, 0, null, null)
        );

        root = new SuffixTreeNode(this, null, null, null, null, -1, -1);

        if (string.length() == 0) return;

        root.addChar(0);
        SuffixTreeNode leaf = root.getLeftChild();

        states.addElement(
            new SuffixTreeState(-1, 0,  root, root)
        );

        for (int i = 0; i < string.length() - 1; i++) {
            phase = i;
            extension = 0;

            states.addElement(
                new SuffixTreeState(i, 0,  root, root)
            );

            SuffixTreeNode node = leaf;
            SuffixTreeNode lastNode = null;

            String activeSubstring = "";

            state = new SuffixTreeState(i, 0, root, node);
            state.selectNode(leaf);
            states.addElement(state);

            node.addChar(i + 1);
            int curpos = i + 1;

            state = new SuffixTreeState(i, 0, root, node);
            state.selectNode(leaf);
            state.selectEdge(leaf);
            states.addElement(state);

            for (int j = 1; j <= i + 1; j++) {
                extension = j;

                state =  new SuffixTreeState(i, j, root, node);
                state.selectNode(node);
                states.addElement(state);

                int len = 0;
                SuffixTreeNode link = node.getSuffixLink();

                if (link == null && !node.isRoot()) {
                    len = node.getLength();
                    activeSubstring = node.getString();
                    if (node.isLeaf()) {
                        activeSubstring = activeSubstring.substring(0,
                                activeSubstring.length() - 1);
                    }

                    state = new SuffixTreeState(i, j, root, node.getParent());
                    state.selectNode(node);                    
                    state.selectNode(node.getParent());
                    state.addReverseArrow(node);
                    state.selectEdge(node);
                    state.setActiveSubstring(activeSubstring);
                    states.addElement(state);

                    curpos = curpos - len;
                    node = node.getParent();
                }

                if (node.getSuffixLink() != null) {
                    state = new SuffixTreeState(i, j, root, node);
                    state.selectLink(node);
                    state.selectNode(node);
                    state.selectNode(node.getSuffixLink());
                    state.setActiveSubstring(activeSubstring);
                    states.addElement(state);

                    node = node.getSuffixLink();

                    if (activeSubstring.length() != 0) {
                        state = new SuffixTreeState(i, j, root, node);
                        state.selectNode(node);
                        state.setActiveSubstring(activeSubstring);
                        states.addElement(state);
                    }

                    if (node.isRoot()) {
                        curpos = j - 1;
                        activeSubstring = getString().substring(j, i + 1);
                    }
                } else {
                    curpos = j - 1;
                    activeSubstring = getString().substring(j, i + 1);

                    state = new SuffixTreeState(i, j, root, node);
                    state.selectNode(root);
                    state.setActiveSubstring(activeSubstring);
                    states.addElement(state);
                }

                while (true) {
                    SuffixTreeNode child = node.getChild(curpos + 1);

                    if (curpos == i) {
                        node.addChar(i + 1);
                        if (node.isLeaf()) {
                            curpos++;

                            state = new SuffixTreeState(i, j, root, node);
                            state.selectNode(node);
                            state.selectEdge(node);
                            state.setSpecialState(SuffixTreeState.STATE_ADD_CHAR_LEAF);
                            states.addElement(state);
                        } else if (child == null) {
                            state = new SuffixTreeState(i, j, root, node);
                            state.selectNode(node);
                            SuffixTreeNode newChild = node.getChild(curpos + 1);
                            state.selectNode(newChild);
                            state.selectEdge(newChild);
                            state.setSpecialState(SuffixTreeState.STATE_ADD_CHAR_NEW_LEAF);
                            states.addElement(state);
                        } else {
                            state = new SuffixTreeState(i, j, root, node);
                            state.selectNode(node);
                            state.selectEdge(child);
                            states.addElement(state);
                        }

                        if (lastNode != null) {
                            lastNode.setSuffixLink(node);

                            state = new SuffixTreeState(i, j, root, node);
                            state.selectNode(lastNode);
                            state.selectLink(lastNode);
                            state.selectNode(node);
                            state.setSpecialState(SuffixTreeState.STATE_LINK_ADDED);
                            states.addElement(state);
                        }
                        lastNode = null;
                        activeSubstring = "";
                        break;
                    }

                    if (curpos + child.getLength() > i) {
                        if (string.charAt(i - curpos + child.getBeginIndex()) ==
                                string.charAt(i + 1)) {
                            activeSubstring = string.substring(child.getBeginIndex(),
                                    child.getBeginIndex() + i - curpos);

                            state = new SuffixTreeState(i, j, root, node);
                            state.setActiveSubstring(activeSubstring);
                            state.selectNode(node);
                            state.selectEdge(child);
                            states.addElement(state);
                            break;
                        }

                        state = new SuffixTreeState(i, j, root, child);
                        state.selectNode(node);
                        state.selectNode(child);
                        state.selectEdge(child);
                        state.setActiveSubstring(activeSubstring);
                        state.setSpecialState(SuffixTreeState.STATE_ADD_CHAR_SPLIT_EDGE);
                        states.addElement(state);

                        SuffixTreeNode leftSibling = child.getLeftSibling();
                        if (leftSibling != null)
                            leftSibling.setRightSibling(child.getRightSibling());
                        else
                            node.setLeftChild(child.getRightSibling());
                        child.setRightSibling(null);

                        node.setLeftChild(
                                new SuffixTreeNode(this, node, child, node.getLeftChild(),
                                        null, child.getBeginIndex(),
                                        i - curpos + child.getBeginIndex())
                        );
                        child.setBeginIndex(i - curpos + child.getBeginIndex());
                        child.setParent(node.getLeftChild());

                        curpos = i;
                        node = node.getLeftChild();
                        node.addChar(i + 1);

                        state = new SuffixTreeState(i, j, root, node);
                        state.selectNode(node);
                        SuffixTreeNode newChild = node.getChild(i + 1);
                        state.selectNode(newChild);
                        state.selectEdge(newChild);
                        states.addElement(state);

                        state = new SuffixTreeState(i, j, root, node);
                        state.selectNode(node);
                        states.addElement(state);

                        if (lastNode != null) {
                            lastNode.setSuffixLink(node);

                            state = new SuffixTreeState(i, j, root, node);
                            state.selectNode(lastNode);
                            state.selectLink(lastNode);
                            state.selectNode(node);
                            state.setSpecialState(SuffixTreeState.STATE_LINK_ADDED);
                            states.addElement(state);
                        }

                        lastNode = node;
                        activeSubstring = "";

                        break;
                    }

                    state = new SuffixTreeState(i, j, root, child);
                    state.selectNode(node);
                    state.selectNode(child);
                    state.selectEdge(child);
                    state.addArrow(child);

                    activeSubstring = activeSubstring.substring(child.getLength(),
                            activeSubstring.length());
                    state.setActiveSubstring(activeSubstring);
                    state.setSpecialState(SuffixTreeState.STATE_DESCENT);
                    states.addElement(state);

                    node = child;
                    curpos += child.getLength();
                }

                state = new SuffixTreeState(i, j, root, node);
                state.selectNode(node);
                states.addElement(state);
            }
            states.addElement(new SuffixTreeState(-2, -2, root, node));
        }
        states.addElement(
            new SuffixTreeState(-2, -2, root, root)
        );
        if (Options.DEBUG) {
            System.out.println("SuffixTreeData.buildTree() finished");
            System.out.println("Generated " + states.size() + " states");
        }
    }

	/**
	 * Returns string of this tree.
     *
     * @return string of this tree.
     */
	public String getString() {
		return string;
	}

	/**
	 * Returns root node of this tree.
	 *
	 * @return	root node of this tree.
	 */
	public SuffixTreeNode getRoot() {
    	return root;
	}

    /**
     * Returns current algorithm state.
     *
     * @return current state.
     */
    public SuffixTreeState getState() {
        return (SuffixTreeState) states.elementAt(step);
    }

    /**
     * Returns algorithm state during next step.
     *
     * @return algorithm state during next step.
     */
    public SuffixTreeState getNextState() {
        return (SuffixTreeState) states.elementAt(step + 1);
    }

    /**
     * Performs next step.
     */
    public void nextStep() {
        step++;
    }

    /**
     * Returns to previous step.
     */
    public void previousStep() {
        step--;
    }

    /**
     * Returns string representation of this tree.
     *
     * @return string representation of this tree.
     */
    public String toString() {
        return root.toString();
    }

    int getExtension() {
        return extension;
    }

    int getPhase() {
        return phase;
    }
}
