package ru.ifmo.vizi.ukkonen;

import ru.ifmo.vizi.ukkonen.SuffixTree.SuffixTreeState;
import ru.ifmo.vizi.base.auto.*;
import java.util.Locale;

public final class Ukkonen extends BaseAutomataWithListener {
    /**
      * .
      */
    public final Data d = new Data();

    /**
      *   
      */
    public Ukkonen(Locale locale) {
        super("ru.ifmo.vizi.ukkonen.Comments", locale); 
        init(new Main(), d); 
    }

    /**
      * .
      */
    public final class Data {
        /**
          * Counter variable for phase loop.
          */
        public int i;

        /**
          * Counter variable for extension loop.
          */
        public int j;

        /**
          * Applet instance.
          */
        public UkkonenVisualizer visualizer;

        public String toString() {
            return visualizer.getSuffixTree().toString();
        }
    }

    /**
      * Builds suffix tree.
      */
    private final class Main implements Automata {
        /**
          *   .
          */
        private final int START_STATE = 0;

        /**
          *   .
          */
        private final int END_STATE = 56;

        /**
          *  .
          */
        private final String[] descriptions = new String[]{" ", "Description of the algorithm", "Description of the algorithm", "Description of the algorithm", "Description of the algorithm", "Adding of the sentinel character", "First phase", "Phase loop counter initialization", "Phase loop", "Begin of the phase", "Selecting leaf as current node", "First extension", "Initialization of the extension loop", "", "Extension begin", "Checks if current node has suffix link", "Checks if current node has suffix link ()", "Checks if we go up from leaf", "Checks if we go up from leaf ()", "There is no suffix link from current node", "There is no suffix link from current node", "Checks if current node has suffix link", "Checks if current node has suffix link ()", "We are in root node", "We are in root node ()", "Root node", "Root node", "Suffix link found", "Checks length of the active substring.", "Checks length of the active substring. ()", "We've passed the suffix link.", "Descent", "Checks if the active substrng if of zero lenght", "Checks if the active substrng if of zero lenght ()", "Descent step", "Descent step", "Checks where we need to append new character.", "Checks where we need to append new character. ()", "We''re in leaf.", "Has new leaf been added.", "Has new leaf been added. ()", "Added new leaf.", "Has edge been split.", "Has edge been split. ()", "We have to split an edge.", "New leaf added.", "We should create a new suffix link during next step.", "Our substring is already in the tree.", "Checks if suffix link has been created", "Checks if suffix link has been created ()", "A suffix link has been created in the tree.", "Extension end", "Extension loop increment", "Phase end", "Increment of the phase loop counter", "End of the algorithm", " "};

        /**
          *   .
          */
        private int state;

        /**
          *   .
          */
        private Automata child;

        /**
          *    .
          */
        public void toStart() {
            state = START_STATE; 
            child = null; 
        }

        /**
          *    .
          */
        public void toEnd() {
            state = END_STATE; 
            child = null; 
        }

        /**
          *      .
          */
        public boolean isAtStart() {
            return state == START_STATE; 
        }

        /**
          *      .
          */
        public boolean isAtEnd() {
            return state == END_STATE; 
        }

        /**
          *   .
          */
        public int getStep() {
            return step; 
        }

        /**
          *    .
          */
        public void stepForward(int level) {
            do {
                step++; 
                //    
                switch (state) {
                    case START_STATE: { //  
                        state = 1; // Description of the algorithm
                        break;
                    }
                    case 1: { // Description of the algorithm
                        state = 2; // Description of the algorithm
                        break;
                    }
                    case 2: { // Description of the algorithm
                        state = 3; // Description of the algorithm
                        break;
                    }
                    case 3: { // Description of the algorithm
                        state = 4; // Description of the algorithm
                        break;
                    }
                    case 4: { // Description of the algorithm
                        state = 5; // Adding of the sentinel character
                        break;
                    }
                    case 5: { // Adding of the sentinel character
                        state = 6; // First phase
                        break;
                    }
                    case 6: { // First phase
                        state = 7; // Phase loop counter initialization
                        break;
                    }
                    case 7: { // Phase loop counter initialization
                        stack.pushBoolean(false); 
                        state = 8; // Phase loop
                        break;
                    }
                    case 8: { // Phase loop
                        if (d.i < d.visualizer.getString().length()) {
                            state = 9; // Begin of the phase
                        } else {
                            state = 55; // End of the algorithm
                        }
                        break;
                    }
                    case 9: { // Begin of the phase
                        state = 10; // Selecting leaf as current node
                        break;
                    }
                    case 10: { // Selecting leaf as current node
                        state = 11; // First extension
                        break;
                    }
                    case 11: { // First extension
                        state = 12; // Initialization of the extension loop
                        break;
                    }
                    case 12: { // Initialization of the extension loop
                        stack.pushBoolean(false); 
                        state = 13; // 
                        break;
                    }
                    case 13: { // 
                        if (d.j <= d.i) {
                            state = 14; // Extension begin
                        } else {
                            state = 53; // Phase end
                        }
                        break;
                    }
                    case 14: { // Extension begin
                        state = 15; // Checks if current node has suffix link
                        break;
                    }
                    case 15: { // Checks if current node has suffix link
                        if (!d.visualizer.getState().getCurrentNode().hasSuffixLink() &&                           !d.visualizer.getState().getCurrentNode().isRoot()) {
                            state = 17; // Checks if we go up from leaf
                        } else {
                            stack.pushBoolean(false); 
                            state = 16; // Checks if current node has suffix link ()
                        }
                        break;
                    }
                    case 16: { // Checks if current node has suffix link ()
                        state = 21; // Checks if current node has suffix link
                        break;
                    }
                    case 17: { // Checks if we go up from leaf
                        if (d.visualizer.getState().getCurrentNode().isLeaf()) {
                            state = 19; // There is no suffix link from current node
                        } else {
                            state = 20; // There is no suffix link from current node
                        }
                        break;
                    }
                    case 18: { // Checks if we go up from leaf ()
                        stack.pushBoolean(true); 
                        state = 16; // Checks if current node has suffix link ()
                        break;
                    }
                    case 19: { // There is no suffix link from current node
                        stack.pushBoolean(true); 
                        state = 18; // Checks if we go up from leaf ()
                        break;
                    }
                    case 20: { // There is no suffix link from current node
                        stack.pushBoolean(false); 
                        state = 18; // Checks if we go up from leaf ()
                        break;
                    }
                    case 21: { // Checks if current node has suffix link
                        if (!d.visualizer.getState().getCurrentNode().hasSuffixLink()) {
                            state = 23; // We are in root node
                        } else {
                            state = 27; // Suffix link found
                        }
                        break;
                    }
                    case 22: { // Checks if current node has suffix link ()
                        stack.pushBoolean(false); 
                        state = 31; // Descent
                        break;
                    }
                    case 23: { // We are in root node
                        if (d.j == d.i) {
                            state = 25; // Root node
                        } else {
                            state = 26; // Root node
                        }
                        break;
                    }
                    case 24: { // We are in root node ()
                        stack.pushBoolean(true); 
                        state = 22; // Checks if current node has suffix link ()
                        break;
                    }
                    case 25: { // Root node
                        stack.pushBoolean(true); 
                        state = 24; // We are in root node ()
                        break;
                    }
                    case 26: { // Root node
                        stack.pushBoolean(false); 
                        state = 24; // We are in root node ()
                        break;
                    }
                    case 27: { // Suffix link found
                        state = 28; // Checks length of the active substring.
                        break;
                    }
                    case 28: { // Checks length of the active substring.
                        if (d.visualizer.getState().getActiveSubstring().length() != 0) {
                            state = 30; // We've passed the suffix link.
                        } else {
                            stack.pushBoolean(false); 
                            state = 29; // Checks length of the active substring. ()
                        }
                        break;
                    }
                    case 29: { // Checks length of the active substring. ()
                        stack.pushBoolean(false); 
                        state = 22; // Checks if current node has suffix link ()
                        break;
                    }
                    case 30: { // We've passed the suffix link.
                        stack.pushBoolean(true); 
                        state = 29; // Checks length of the active substring. ()
                        break;
                    }
                    case 31: { // Descent
                        if (d.visualizer.getNextState().getSpecialState() ==                           SuffixTreeState.STATE_DESCENT) {
                            state = 32; // Checks if the active substrng if of zero lenght
                        } else {
                            state = 36; // Checks where we need to append new character.
                        }
                        break;
                    }
                    case 32: { // Checks if the active substrng if of zero lenght
                        if (d.visualizer.getNextState().getActiveSubstring().length() == 0) {
                            state = 34; // Descent step
                        } else {
                            state = 35; // Descent step
                        }
                        break;
                    }
                    case 33: { // Checks if the active substrng if of zero lenght ()
                        stack.pushBoolean(true); 
                        state = 31; // Descent
                        break;
                    }
                    case 34: { // Descent step
                        stack.pushBoolean(true); 
                        state = 33; // Checks if the active substrng if of zero lenght ()
                        break;
                    }
                    case 35: { // Descent step
                        stack.pushBoolean(false); 
                        state = 33; // Checks if the active substrng if of zero lenght ()
                        break;
                    }
                    case 36: { // Checks where we need to append new character.
                        if (d.visualizer.getNextState().getSpecialState() ==                           SuffixTreeState.STATE_ADD_CHAR_LEAF) {
                            state = 38; // We''re in leaf.
                        } else {
                            state = 39; // Has new leaf been added.
                        }
                        break;
                    }
                    case 37: { // Checks where we need to append new character. ()
                        state = 48; // Checks if suffix link has been created
                        break;
                    }
                    case 38: { // We''re in leaf.
                        stack.pushBoolean(true); 
                        state = 37; // Checks where we need to append new character. ()
                        break;
                    }
                    case 39: { // Has new leaf been added.
                        if (d.visualizer.getNextState().getSpecialState() ==                                 SuffixTreeState.STATE_ADD_CHAR_NEW_LEAF) {
                            state = 41; // Added new leaf.
                        } else {
                            state = 42; // Has edge been split.
                        }
                        break;
                    }
                    case 40: { // Has new leaf been added. ()
                        stack.pushBoolean(false); 
                        state = 37; // Checks where we need to append new character. ()
                        break;
                    }
                    case 41: { // Added new leaf.
                        stack.pushBoolean(true); 
                        state = 40; // Has new leaf been added. ()
                        break;
                    }
                    case 42: { // Has edge been split.
                        if (d.visualizer.getNextState().getSpecialState() ==                                         SuffixTreeState.STATE_ADD_CHAR_SPLIT_EDGE) {
                            state = 44; // We have to split an edge.
                        } else {
                            state = 47; // Our substring is already in the tree.
                        }
                        break;
                    }
                    case 43: { // Has edge been split. ()
                        stack.pushBoolean(false); 
                        state = 40; // Has new leaf been added. ()
                        break;
                    }
                    case 44: { // We have to split an edge.
                        state = 45; // New leaf added.
                        break;
                    }
                    case 45: { // New leaf added.
                        state = 46; // We should create a new suffix link during next step.
                        break;
                    }
                    case 46: { // We should create a new suffix link during next step.
                        stack.pushBoolean(true); 
                        state = 43; // Has edge been split. ()
                        break;
                    }
                    case 47: { // Our substring is already in the tree.
                        stack.pushBoolean(false); 
                        state = 43; // Has edge been split. ()
                        break;
                    }
                    case 48: { // Checks if suffix link has been created
                        if (d.visualizer.getNextState().getSpecialState() ==                           SuffixTreeState.STATE_LINK_ADDED) {
                            state = 50; // A suffix link has been created in the tree.
                        } else {
                            stack.pushBoolean(false); 
                            state = 49; // Checks if suffix link has been created ()
                        }
                        break;
                    }
                    case 49: { // Checks if suffix link has been created ()
                        state = 51; // Extension end
                        break;
                    }
                    case 50: { // A suffix link has been created in the tree.
                        stack.pushBoolean(true); 
                        state = 49; // Checks if suffix link has been created ()
                        break;
                    }
                    case 51: { // Extension end
                        state = 52; // Extension loop increment
                        break;
                    }
                    case 52: { // Extension loop increment
                        stack.pushBoolean(true); 
                        state = 13; // 
                        break;
                    }
                    case 53: { // Phase end
                        state = 54; // Increment of the phase loop counter
                        break;
                    }
                    case 54: { // Increment of the phase loop counter
                        stack.pushBoolean(true); 
                        state = 8; // Phase loop
                        break;
                    }
                    case 55: { // End of the algorithm
                        state = END_STATE; 
                        break;
                    }
                }

                //    
                switch (state) {
                    case 1: { // Description of the algorithm
                        break;
                    }
                    case 2: { // Description of the algorithm
                        break;
                    }
                    case 3: { // Description of the algorithm
                        break;
                    }
                    case 4: { // Description of the algorithm
                        break;
                    }
                    case 5: { // Adding of the sentinel character
                        d.visualizer.addSentinelCharacter();
                        break;
                    }
                    case 6: { // First phase
                        d.visualizer.nextStep();
                        break;
                    }
                    case 7: { // Phase loop counter initialization
                        d.i = 1;
                        break;
                    }
                    case 8: { // Phase loop
                        break;
                    }
                    case 9: { // Begin of the phase
                        d.visualizer.nextStep();
                        break;
                    }
                    case 10: { // Selecting leaf as current node
                        d.visualizer.nextStep();
                        break;
                    }
                    case 11: { // First extension
                        d.visualizer.nextStep();
                        break;
                    }
                    case 12: { // Initialization of the extension loop
                        d.j = 1;
                        break;
                    }
                    case 13: { // 
                        break;
                    }
                    case 14: { // Extension begin
                        d.visualizer.nextStep();
                        break;
                    }
                    case 15: { // Checks if current node has suffix link
                        break;
                    }
                    case 16: { // Checks if current node has suffix link ()
                        break;
                    }
                    case 17: { // Checks if we go up from leaf
                        break;
                    }
                    case 18: { // Checks if we go up from leaf ()
                        break;
                    }
                    case 19: { // There is no suffix link from current node
                        d.visualizer.nextStep();
                        break;
                    }
                    case 20: { // There is no suffix link from current node
                        d.visualizer.nextStep();
                        break;
                    }
                    case 21: { // Checks if current node has suffix link
                        break;
                    }
                    case 22: { // Checks if current node has suffix link ()
                        break;
                    }
                    case 23: { // We are in root node
                        break;
                    }
                    case 24: { // We are in root node ()
                        break;
                    }
                    case 25: { // Root node
                        d.visualizer.nextStep();
                        break;
                    }
                    case 26: { // Root node
                        d.visualizer.nextStep();
                        break;
                    }
                    case 27: { // Suffix link found
                        d.visualizer.nextStep();
                        break;
                    }
                    case 28: { // Checks length of the active substring.
                        break;
                    }
                    case 29: { // Checks length of the active substring. ()
                        break;
                    }
                    case 30: { // We've passed the suffix link.
                        d.visualizer.nextStep();
                        break;
                    }
                    case 31: { // Descent
                        break;
                    }
                    case 32: { // Checks if the active substrng if of zero lenght
                        break;
                    }
                    case 33: { // Checks if the active substrng if of zero lenght ()
                        break;
                    }
                    case 34: { // Descent step
                        d.visualizer.nextStep();
                        break;
                    }
                    case 35: { // Descent step
                        d.visualizer.nextStep();
                        break;
                    }
                    case 36: { // Checks where we need to append new character.
                        break;
                    }
                    case 37: { // Checks where we need to append new character. ()
                        break;
                    }
                    case 38: { // We''re in leaf.
                        d.visualizer.nextStep();
                        break;
                    }
                    case 39: { // Has new leaf been added.
                        break;
                    }
                    case 40: { // Has new leaf been added. ()
                        break;
                    }
                    case 41: { // Added new leaf.
                        d.visualizer.nextStep();
                        break;
                    }
                    case 42: { // Has edge been split.
                        break;
                    }
                    case 43: { // Has edge been split. ()
                        break;
                    }
                    case 44: { // We have to split an edge.
                        d.visualizer.nextStep();
                        break;
                    }
                    case 45: { // New leaf added.
                        d.visualizer.nextStep();
                        break;
                    }
                    case 46: { // We should create a new suffix link during next step.
                        d.visualizer.nextStep();
                        break;
                    }
                    case 47: { // Our substring is already in the tree.
                        d.visualizer.nextStep();
                        break;
                    }
                    case 48: { // Checks if suffix link has been created
                        break;
                    }
                    case 49: { // Checks if suffix link has been created ()
                        break;
                    }
                    case 50: { // A suffix link has been created in the tree.
                        d.visualizer.nextStep();
                        break;
                    }
                    case 51: { // Extension end
                        d.visualizer.nextStep();
                        break;
                    }
                    case 52: { // Extension loop increment
                        d.j++;
                        break;
                    }
                    case 53: { // Phase end
                        d.visualizer.nextStep();
                        break;
                    }
                    case 54: { // Increment of the phase loop counter
                        d.i++;
                        break;
                    }
                    case 55: { // End of the algorithm
                        d.visualizer.nextStep();
                        break;
                    }
                }
            } while (!isInteresting(level));
        }

        /**
          *    .
          */
        public void stepBackward(int level) {
            do {
                //     
                switch (state) {
                    case 1: { // Description of the algorithm
                        break;
                    }
                    case 2: { // Description of the algorithm
                        break;
                    }
                    case 3: { // Description of the algorithm
                        break;
                    }
                    case 4: { // Description of the algorithm
                        break;
                    }
                    case 5: { // Adding of the sentinel character
                        d.visualizer.removeSentinelCharacter();
                        break;
                    }
                    case 6: { // First phase
                        d.visualizer.previousStep();
                        break;
                    }
                    case 7: { // Phase loop counter initialization
                        break;
                    }
                    case 8: { // Phase loop
                        break;
                    }
                    case 9: { // Begin of the phase
                        d.visualizer.previousStep();
                        break;
                    }
                    case 10: { // Selecting leaf as current node
                        d.visualizer.previousStep();
                        break;
                    }
                    case 11: { // First extension
                        d.visualizer.previousStep();
                        break;
                    }
                    case 12: { // Initialization of the extension loop
                        break;
                    }
                    case 13: { // 
                        break;
                    }
                    case 14: { // Extension begin
                        d.visualizer.previousStep();
                        break;
                    }
                    case 15: { // Checks if current node has suffix link
                        break;
                    }
                    case 16: { // Checks if current node has suffix link ()
                        break;
                    }
                    case 17: { // Checks if we go up from leaf
                        break;
                    }
                    case 18: { // Checks if we go up from leaf ()
                        break;
                    }
                    case 19: { // There is no suffix link from current node
                        d.visualizer.previousStep();
                        break;
                    }
                    case 20: { // There is no suffix link from current node
                        d.visualizer.previousStep();
                        break;
                    }
                    case 21: { // Checks if current node has suffix link
                        break;
                    }
                    case 22: { // Checks if current node has suffix link ()
                        break;
                    }
                    case 23: { // We are in root node
                        break;
                    }
                    case 24: { // We are in root node ()
                        break;
                    }
                    case 25: { // Root node
                        d.visualizer.previousStep();
                        break;
                    }
                    case 26: { // Root node
                        d.visualizer.previousStep();
                        break;
                    }
                    case 27: { // Suffix link found
                        d.visualizer.previousStep();
                        break;
                    }
                    case 28: { // Checks length of the active substring.
                        break;
                    }
                    case 29: { // Checks length of the active substring. ()
                        break;
                    }
                    case 30: { // We've passed the suffix link.
                        d.visualizer.previousStep();
                        break;
                    }
                    case 31: { // Descent
                        break;
                    }
                    case 32: { // Checks if the active substrng if of zero lenght
                        break;
                    }
                    case 33: { // Checks if the active substrng if of zero lenght ()
                        break;
                    }
                    case 34: { // Descent step
                        d.visualizer.previousStep();
                        break;
                    }
                    case 35: { // Descent step
                        d.visualizer.previousStep();
                        break;
                    }
                    case 36: { // Checks where we need to append new character.
                        break;
                    }
                    case 37: { // Checks where we need to append new character. ()
                        break;
                    }
                    case 38: { // We''re in leaf.
                        d.visualizer.previousStep();
                        break;
                    }
                    case 39: { // Has new leaf been added.
                        break;
                    }
                    case 40: { // Has new leaf been added. ()
                        break;
                    }
                    case 41: { // Added new leaf.
                        d.visualizer.previousStep();
                        break;
                    }
                    case 42: { // Has edge been split.
                        break;
                    }
                    case 43: { // Has edge been split. ()
                        break;
                    }
                    case 44: { // We have to split an edge.
                        d.visualizer.previousStep();
                        break;
                    }
                    case 45: { // New leaf added.
                        d.visualizer.previousStep();
                        break;
                    }
                    case 46: { // We should create a new suffix link during next step.
                        d.visualizer.previousStep();
                        break;
                    }
                    case 47: { // Our substring is already in the tree.
                        d.visualizer.previousStep();
                        break;
                    }
                    case 48: { // Checks if suffix link has been created
                        break;
                    }
                    case 49: { // Checks if suffix link has been created ()
                        break;
                    }
                    case 50: { // A suffix link has been created in the tree.
                        d.visualizer.previousStep();
                        break;
                    }
                    case 51: { // Extension end
                        d.visualizer.previousStep();
                        break;
                    }
                    case 52: { // Extension loop increment
                        d.j--;
                        break;
                    }
                    case 53: { // Phase end
                        d.visualizer.previousStep();
                        d.j = d.i + 1;
                        break;
                    }
                    case 54: { // Increment of the phase loop counter
                        d.i--;
                        break;
                    }
                    case 55: { // End of the algorithm
                        d.visualizer.previousStep();
                        break;
                    }
                }

                //    
                switch (state) {
                    case 1: { // Description of the algorithm
                        state = START_STATE; 
                        break;
                    }
                    case 2: { // Description of the algorithm
                        state = 1; // Description of the algorithm
                        break;
                    }
                    case 3: { // Description of the algorithm
                        state = 2; // Description of the algorithm
                        break;
                    }
                    case 4: { // Description of the algorithm
                        state = 3; // Description of the algorithm
                        break;
                    }
                    case 5: { // Adding of the sentinel character
                        state = 4; // Description of the algorithm
                        break;
                    }
                    case 6: { // First phase
                        state = 5; // Adding of the sentinel character
                        break;
                    }
                    case 7: { // Phase loop counter initialization
                        state = 6; // First phase
                        break;
                    }
                    case 8: { // Phase loop
                        if (stack.popBoolean()) {
                            state = 54; // Increment of the phase loop counter
                        } else {
                            state = 7; // Phase loop counter initialization
                        }
                        break;
                    }
                    case 9: { // Begin of the phase
                        state = 8; // Phase loop
                        break;
                    }
                    case 10: { // Selecting leaf as current node
                        state = 9; // Begin of the phase
                        break;
                    }
                    case 11: { // First extension
                        state = 10; // Selecting leaf as current node
                        break;
                    }
                    case 12: { // Initialization of the extension loop
                        state = 11; // First extension
                        break;
                    }
                    case 13: { // 
                        if (stack.popBoolean()) {
                            state = 52; // Extension loop increment
                        } else {
                            state = 12; // Initialization of the extension loop
                        }
                        break;
                    }
                    case 14: { // Extension begin
                        state = 13; // 
                        break;
                    }
                    case 15: { // Checks if current node has suffix link
                        state = 14; // Extension begin
                        break;
                    }
                    case 16: { // Checks if current node has suffix link ()
                        if (stack.popBoolean()) {
                            state = 18; // Checks if we go up from leaf ()
                        } else {
                            state = 15; // Checks if current node has suffix link
                        }
                        break;
                    }
                    case 17: { // Checks if we go up from leaf
                        state = 15; // Checks if current node has suffix link
                        break;
                    }
                    case 18: { // Checks if we go up from leaf ()
                        if (stack.popBoolean()) {
                            state = 19; // There is no suffix link from current node
                        } else {
                            state = 20; // There is no suffix link from current node
                        }
                        break;
                    }
                    case 19: { // There is no suffix link from current node
                        state = 17; // Checks if we go up from leaf
                        break;
                    }
                    case 20: { // There is no suffix link from current node
                        state = 17; // Checks if we go up from leaf
                        break;
                    }
                    case 21: { // Checks if current node has suffix link
                        state = 16; // Checks if current node has suffix link ()
                        break;
                    }
                    case 22: { // Checks if current node has suffix link ()
                        if (stack.popBoolean()) {
                            state = 24; // We are in root node ()
                        } else {
                            state = 29; // Checks length of the active substring. ()
                        }
                        break;
                    }
                    case 23: { // We are in root node
                        state = 21; // Checks if current node has suffix link
                        break;
                    }
                    case 24: { // We are in root node ()
                        if (stack.popBoolean()) {
                            state = 25; // Root node
                        } else {
                            state = 26; // Root node
                        }
                        break;
                    }
                    case 25: { // Root node
                        state = 23; // We are in root node
                        break;
                    }
                    case 26: { // Root node
                        state = 23; // We are in root node
                        break;
                    }
                    case 27: { // Suffix link found
                        state = 21; // Checks if current node has suffix link
                        break;
                    }
                    case 28: { // Checks length of the active substring.
                        state = 27; // Suffix link found
                        break;
                    }
                    case 29: { // Checks length of the active substring. ()
                        if (stack.popBoolean()) {
                            state = 30; // We've passed the suffix link.
                        } else {
                            state = 28; // Checks length of the active substring.
                        }
                        break;
                    }
                    case 30: { // We've passed the suffix link.
                        state = 28; // Checks length of the active substring.
                        break;
                    }
                    case 31: { // Descent
                        if (stack.popBoolean()) {
                            state = 33; // Checks if the active substrng if of zero lenght ()
                        } else {
                            state = 22; // Checks if current node has suffix link ()
                        }
                        break;
                    }
                    case 32: { // Checks if the active substrng if of zero lenght
                        state = 31; // Descent
                        break;
                    }
                    case 33: { // Checks if the active substrng if of zero lenght ()
                        if (stack.popBoolean()) {
                            state = 34; // Descent step
                        } else {
                            state = 35; // Descent step
                        }
                        break;
                    }
                    case 34: { // Descent step
                        state = 32; // Checks if the active substrng if of zero lenght
                        break;
                    }
                    case 35: { // Descent step
                        state = 32; // Checks if the active substrng if of zero lenght
                        break;
                    }
                    case 36: { // Checks where we need to append new character.
                        state = 31; // Descent
                        break;
                    }
                    case 37: { // Checks where we need to append new character. ()
                        if (stack.popBoolean()) {
                            state = 38; // We''re in leaf.
                        } else {
                            state = 40; // Has new leaf been added. ()
                        }
                        break;
                    }
                    case 38: { // We''re in leaf.
                        state = 36; // Checks where we need to append new character.
                        break;
                    }
                    case 39: { // Has new leaf been added.
                        state = 36; // Checks where we need to append new character.
                        break;
                    }
                    case 40: { // Has new leaf been added. ()
                        if (stack.popBoolean()) {
                            state = 41; // Added new leaf.
                        } else {
                            state = 43; // Has edge been split. ()
                        }
                        break;
                    }
                    case 41: { // Added new leaf.
                        state = 39; // Has new leaf been added.
                        break;
                    }
                    case 42: { // Has edge been split.
                        state = 39; // Has new leaf been added.
                        break;
                    }
                    case 43: { // Has edge been split. ()
                        if (stack.popBoolean()) {
                            state = 46; // We should create a new suffix link during next step.
                        } else {
                            state = 47; // Our substring is already in the tree.
                        }
                        break;
                    }
                    case 44: { // We have to split an edge.
                        state = 42; // Has edge been split.
                        break;
                    }
                    case 45: { // New leaf added.
                        state = 44; // We have to split an edge.
                        break;
                    }
                    case 46: { // We should create a new suffix link during next step.
                        state = 45; // New leaf added.
                        break;
                    }
                    case 47: { // Our substring is already in the tree.
                        state = 42; // Has edge been split.
                        break;
                    }
                    case 48: { // Checks if suffix link has been created
                        state = 37; // Checks where we need to append new character. ()
                        break;
                    }
                    case 49: { // Checks if suffix link has been created ()
                        if (stack.popBoolean()) {
                            state = 50; // A suffix link has been created in the tree.
                        } else {
                            state = 48; // Checks if suffix link has been created
                        }
                        break;
                    }
                    case 50: { // A suffix link has been created in the tree.
                        state = 48; // Checks if suffix link has been created
                        break;
                    }
                    case 51: { // Extension end
                        state = 49; // Checks if suffix link has been created ()
                        break;
                    }
                    case 52: { // Extension loop increment
                        state = 51; // Extension end
                        break;
                    }
                    case 53: { // Phase end
                        state = 13; // 
                        break;
                    }
                    case 54: { // Increment of the phase loop counter
                        state = 53; // Phase end
                        break;
                    }
                    case 55: { // End of the algorithm
                        state = 8; // Phase loop
                        break;
                    }
                    case END_STATE: { //  
                        state = 55; // End of the algorithm
                        break;
                    }
                }

                step--; 
            } while (!isInteresting(level));
        }

        /**
          *    .
          */
        public boolean isInteresting(int level) {
            // 
            switch (state) {
                case START_STATE: //  
                    return true; 
                case 1: // Description of the algorithm
                    return level <= 0; 
                case 2: // Description of the algorithm
                    return level <= 0; 
                case 3: // Description of the algorithm
                    return level <= 0; 
                case 4: // Description of the algorithm
                    return level <= 0; 
                case 5: // Adding of the sentinel character
                    return level <= 1; 
                case 6: // First phase
                    return level <= 1; 
                case 7: // Phase loop counter initialization
                    return level <= -1; 
                case 8: // Phase loop
                    return level <= -1; 
                case 9: // Begin of the phase
                    return level <= 1; 
                case 10: // Selecting leaf as current node
                    return level <= 0; 
                case 11: // First extension
                    return level <= 1; 
                case 12: // Initialization of the extension loop
                    return level <= -1; 
                case 13: // 
                    return level <= -1; 
                case 14: // Extension begin
                    return level <= 1; 
                case 15: // Checks if current node has suffix link
                    return level <= -1; 
                case 16: // Checks if current node has suffix link ()
                    return level <= -1; 
                case 17: // Checks if we go up from leaf
                    return level <= -1; 
                case 18: // Checks if we go up from leaf ()
                    return level <= -1; 
                case 19: // There is no suffix link from current node
                    return level <= 0; 
                case 20: // There is no suffix link from current node
                    return level <= 0; 
                case 21: // Checks if current node has suffix link
                    return level <= -1; 
                case 22: // Checks if current node has suffix link ()
                    return level <= -1; 
                case 23: // We are in root node
                    return level <= -1; 
                case 24: // We are in root node ()
                    return level <= -1; 
                case 25: // Root node
                    return level <= 0; 
                case 26: // Root node
                    return level <= 0; 
                case 27: // Suffix link found
                    return level <= 0; 
                case 28: // Checks length of the active substring.
                    return level <= -1; 
                case 29: // Checks length of the active substring. ()
                    return level <= -1; 
                case 30: // We've passed the suffix link.
                    return level <= 0; 
                case 31: // Descent
                    return level <= -1; 
                case 32: // Checks if the active substrng if of zero lenght
                    return level <= -1; 
                case 33: // Checks if the active substrng if of zero lenght ()
                    return level <= -1; 
                case 34: // Descent step
                    return level <= 0; 
                case 35: // Descent step
                    return level <= 0; 
                case 36: // Checks where we need to append new character.
                    return level <= -1; 
                case 37: // Checks where we need to append new character. ()
                    return level <= -1; 
                case 38: // We''re in leaf.
                    return level <= 0; 
                case 39: // Has new leaf been added.
                    return level <= -1; 
                case 40: // Has new leaf been added. ()
                    return level <= -1; 
                case 41: // Added new leaf.
                    return level <= 0; 
                case 42: // Has edge been split.
                    return level <= -1; 
                case 43: // Has edge been split. ()
                    return level <= -1; 
                case 44: // We have to split an edge.
                    return level <= 0; 
                case 45: // New leaf added.
                    return level <= 0; 
                case 46: // We should create a new suffix link during next step.
                    return level <= 0; 
                case 47: // Our substring is already in the tree.
                    return level <= 0; 
                case 48: // Checks if suffix link has been created
                    return level <= -1; 
                case 49: // Checks if suffix link has been created ()
                    return level <= -1; 
                case 50: // A suffix link has been created in the tree.
                    return level <= 0; 
                case 51: // Extension end
                    return level <= 0; 
                case 52: // Extension loop increment
                    return level <= -1; 
                case 53: // Phase end
                    return level <= 0; 
                case 54: // Increment of the phase loop counter
                    return level <= -1; 
                case 55: // End of the algorithm
                    return level <= -1; 
                case END_STATE: //  
                    return true; 
            }

            throw new RuntimeException("isInterest"); 
        }

        /**
          *    
          */
        public String getComment() {
            String comment = ""; 
            Object[] args = null; 
            //  
            switch (state) {
                case START_STATE: { //  
                    comment = Ukkonen.this.getComment("Main.START_STATE"); 
                    break;
                }
                case 1: { // Description of the algorithm
                    comment = Ukkonen.this.getComment("Main.Description1"); 
                    break;
                }
                case 2: { // Description of the algorithm
                    comment = Ukkonen.this.getComment("Main.Description2"); 
                    break;
                }
                case 3: { // Description of the algorithm
                    comment = Ukkonen.this.getComment("Main.Description3"); 
                    break;
                }
                case 4: { // Description of the algorithm
                    comment = Ukkonen.this.getComment("Main.Description4"); 
                    break;
                }
                case 5: { // Adding of the sentinel character
                    comment = Ukkonen.this.getComment("Main.AddSentinelCharacter"); 
                    args = new Object[]{d.visualizer.SENTINEL}; 
                    break;
                }
                case 6: { // First phase
                    comment = Ukkonen.this.getComment("Main.FirstPhase"); 
                    break;
                }
                case 9: { // Begin of the phase
                    comment = Ukkonen.this.getComment("Main.PhaseBegin"); 
                    args = new Object[]{new Integer(d.i), new Integer(d.i + 1),                               d.visualizer.getString().substring(0, d.i + 1),                               d.visualizer.getString().substring(d.i, d.i + 1)}; 
                    break;
                }
                case 10: { // Selecting leaf as current node
                    comment = Ukkonen.this.getComment("Main.SelectLeaf"); 
                    break;
                }
                case 11: { // First extension
                    comment = Ukkonen.this.getComment("Main.FirstExtension"); 
                    args = new Object[]{new Character(d.visualizer.getString().charAt(d.i)),                               d.visualizer.getString().substring(0, d.i + 1)}; 
                    break;
                }
                case 14: { // Extension begin
                    comment = Ukkonen.this.getComment("Main.ExtensionBegin"); 
                    args = new Object[]{d.visualizer.getString().substring(d.j, d.i + 1),                                   new Integer(d.j + 1)}; 
                    break;
                }
                case 19: { // There is no suffix link from current node
                    comment = Ukkonen.this.getComment("Main.NoSuffixLink0"); 
                    break;
                }
                case 20: { // There is no suffix link from current node
                    comment = Ukkonen.this.getComment("Main.NoSuffixLink1"); 
                    break;
                }
                case 25: { // Root node
                    comment = Ukkonen.this.getComment("Main.NoSuffixLink3"); 
                    args = new Object[]{d.visualizer.getString().substring(d.i, d.i + 1)}; 
                    break;
                }
                case 26: { // Root node
                    comment = Ukkonen.this.getComment("Main.NoSuffixLink2"); 
                    args = new Object[]{d.visualizer.getString().substring(d.j, d.i),                                                   d.visualizer.getString().substring(d.i, d.i + 1)}; 
                    break;
                }
                case 27: { // Suffix link found
                    comment = Ukkonen.this.getComment("Main.FoundSuffixLink"); 
                    break;
                }
                case 30: { // We've passed the suffix link.
                    comment = Ukkonen.this.getComment("Main.SuffixLinkEnd"); 
                    args = new Object[]{d.visualizer.getString().substring(d.j, d.i),                                                   d.visualizer.getString().substring(d.i, d.i + 1)}; 
                    break;
                }
                case 34: { // Descent step
                    comment = Ukkonen.this.getComment("Main.DescentStep1"); 
                    args = new Object[]{d.visualizer.getState().getCurrentNode().getString(),                                               d.visualizer.getString().substring(d.i, d.i + 1)}; 
                    break;
                }
                case 35: { // Descent step
                    comment = Ukkonen.this.getComment("Main.DescentStep2"); 
                    args = new Object[]{d.visualizer.getState().getCurrentNode().getString(),                                               d.visualizer.getState().getActiveSubstring()}; 
                    break;
                }
                case 38: { // We''re in leaf.
                    comment = Ukkonen.this.getComment("Main.AppendCharLeaf"); 
                    args = new Object[]{d.visualizer.getString().substring(d.i, d.i + 1)}; 
                    break;
                }
                case 41: { // Added new leaf.
                    comment = Ukkonen.this.getComment("Main.AddNewLeaf"); 
                    args = new Object[]{d.visualizer.getString().substring(d.i, d.i + 1)}; 
                    break;
                }
                case 44: { // We have to split an edge.
                    comment = Ukkonen.this.getComment("Main.SplittingEdge"); 
                    args = new Object[]{d.visualizer.getString().substring(d.i, d.i + 1),                                                           d.visualizer.getState().getActiveSubstring(),                                                           d.visualizer.getState().getCurrentNode().getString().substring(d.visualizer.getState().getActiveSubstring().length(), d.visualizer.getState().getCurrentNode().getString().length())}; 
                    break;
                }
                case 45: { // New leaf added.
                    comment = Ukkonen.this.getComment("Main.SplitEdgeAddLeaf"); 
                    args = new Object[]{d.visualizer.getString().substring(d.i, d.i + 1)}; 
                    break;
                }
                case 46: { // We should create a new suffix link during next step.
                    comment = Ukkonen.this.getComment("Main.SuffixLinkOnNextStep"); 
                    break;
                }
                case 47: { // Our substring is already in the tree.
                    comment = Ukkonen.this.getComment("Main.AlreadyInTree"); 
                    break;
                }
                case 50: { // A suffix link has been created in the tree.
                    comment = Ukkonen.this.getComment("Main.LinkHasBeenCreated"); 
                    break;
                }
                case 51: { // Extension end
                    comment = Ukkonen.this.getComment("Main.ExtensionEnd"); 
                    args = new Object[]{d.visualizer.getString().substring(d.j, d.i + 1)}; 
                    break;
                }
                case 53: { // Phase end
                    comment = Ukkonen.this.getComment("Main.PhaseEnd"); 
                    args = new Object[]{new Integer(d.i)}; 
                    break;
                }
                case END_STATE: { //  
                    comment = Ukkonen.this.getComment("Main.END_STATE"); 
                    break;
                }
            }

            return java.text.MessageFormat.format(comment, args); 
        }

        /**
          *     
          */
        public void drawState() {
            switch (state) {
                case START_STATE: { //  
                    break;
                }
                case 5: { // Adding of the sentinel character
                    d.visualizer.updateTree();
                    break;
                }
                case 6: { // First phase
                    d.visualizer.updateTree();
                    break;
                }
                case 9: { // Begin of the phase
                    d.visualizer.updateTree();
                    break;
                }
                case 10: { // Selecting leaf as current node
                    d.visualizer.updateTree();
                    break;
                }
                case 11: { // First extension
                    d.visualizer.updateTree();
                    break;
                }
                case 14: { // Extension begin
                    d.visualizer.updateTree();
                    break;
                }
                case 19: { // There is no suffix link from current node
                    d.visualizer.updateTree();
                    break;
                }
                case 20: { // There is no suffix link from current node
                    d.visualizer.updateTree();
                    break;
                }
                case 25: { // Root node
                    d.visualizer.updateTree();
                    break;
                }
                case 26: { // Root node
                    d.visualizer.updateTree();
                    break;
                }
                case 27: { // Suffix link found
                    d.visualizer.updateTree();
                    break;
                }
                case 30: { // We've passed the suffix link.
                    d.visualizer.updateTree();
                    break;
                }
                case 34: { // Descent step
                    d.visualizer.updateTree();
                    break;
                }
                case 35: { // Descent step
                    d.visualizer.updateTree();
                    break;
                }
                case 38: { // We''re in leaf.
                    d.visualizer.updateTree();
                    break;
                }
                case 41: { // Added new leaf.
                    d.visualizer.updateTree();
                    break;
                }
                case 44: { // We have to split an edge.
                    d.visualizer.updateTree();
                    break;
                }
                case 45: { // New leaf added.
                    d.visualizer.updateTree();
                    break;
                }
                case 46: { // We should create a new suffix link during next step.
                    d.visualizer.updateTree();
                    break;
                }
                case 47: { // Our substring is already in the tree.
                    d.visualizer.updateTree();
                    break;
                }
                case 50: { // A suffix link has been created in the tree.
                    d.visualizer.updateTree();
                    break;
                }
                case 51: { // Extension end
                    d.visualizer.updateTree();
                    break;
                }
                case 53: { // Phase end
                    d.visualizer.updateTree();
                    break;
                }
                case END_STATE: { //  
                    d.visualizer.updateTree();
                    break;
                }
            }
        }

        public StringBuffer toString(StringBuffer s) {
            s.append("Main ").append(state).append(" "); 
            s.append('('); 
            s.append(descriptions[state]); 
            s.append(")\n"); 
            if (child != null && !child.isAtStart() && !child.isAtEnd()) {
                child.toString(s); 
            }
            return s; 
        }
    }
}
