package ru.ifmo.vizi.ukkonen.ui;

import ru.ifmo.vizi.base.Configuration;
import ru.ifmo.vizi.base.ui.Hinter;

import java.awt.*;
import java.awt.event.ComponentEvent;

/**
 * Simple checkbox with hint.
 * <p>
 * Used configuration parameters:
 * <table border="1">
 *      <tr>
 *          <th>Name</th>
 *          <th>Description</th>
 *      </th>
 *      <tr>
 *          <td><b>name</b></td>
 *          <td>Checkbox caption</td>
 *      </tr>
 *      <tr>
 *          <td><b>name</b>-hint</td>
 *          <td>Checkbox hint</td>
 *      </tr>
 * </table>
 *
 * @author  Igor Ahmetov
 * @version $Id: 0.0$
 */

public class HintedCheckbox extends Checkbox {
    /**
     * Creates a new hinted checkbox without caption and hint.
     */
    protected HintedCheckbox() {
        enableEvents(AWTEvent.ACTION_EVENT_MASK | AWTEvent.COMPONENT_EVENT_MASK);
    }

    /**
     * Creates a new hinted checkbox with caption specified in parameter
     * <link>name</link> and hint specified in parameter <link>name</link>-hint.
     *
     * @param config applet configuration.
     * @param name checkbox name.
     */
    public HintedCheckbox(Configuration config, String name) {
        this();

        setLabel(config.getParameter(name));
        setHint(config.getParameter(name + "-hint"));
    }

    /**
     * Hint to show.
     */
    private String hint;

    /**
     * Sets new hint for this checkbox.
     *
     * @param hint hint to set.
     */
    public void setHint(String hint) {
        this.hint = hint;
        applyHint();
    }

    /**
     * Applies hint, if needed.
     */
    private void applyHint() {
        if (isVisible()) {
            Hinter.applyHint(this, hint);
        } else {
            Hinter.applyHint(this, null);
        }
    }

    /**
     * Invoked when component state changes.
     *
     * @param e the component event
     */
    protected final void processComponentEvent(ComponentEvent e) {
        super.processComponentEvent(e);
        applyHint();
    }
}
