package ru.ifmo.vizi.ukkonen.ui;

import ru.ifmo.vizi.base.Configuration;
import ru.ifmo.vizi.base.ui.Hinter;

import java.awt.*;
import java.awt.event.ComponentEvent;

/**
 * Simple textfield with hint.
 *
 * @author  Igor Ahmetov
 * @version $Id$
 */

public class HintedTextField extends TextField {
    /**
     * Creates a new hinted textfiled without hint,
     * initialized with the specified text to be displayed,
     * and wide enough to hold the specified number of columns.
     *
     * @param string    the text to be displayed.
     * @param columns   the number of columns.
     */
    protected HintedTextField(String string, int columns) {
        super(string, columns);
        enableEvents(AWTEvent.ACTION_EVENT_MASK | AWTEvent.COMPONENT_EVENT_MASK);
    }

    /**
     * Creates a new hinted textfield with hint specified in parameter
     * <link>name</link>-hint, initialized with the specified text
     * to be displayed, and wide enough to hold the specified number of
     * columns.
     *
     * @param config applet configuration.
     * @param name text field name.
     */
    public HintedTextField(Configuration config, String name,
                        String text, int columns) {
        this(text, columns);

        setHint(config.getParameter(name + "-hint"));
    }

    /**
     * Hint to show.
     */
    private String hint;

    /**
     * Sets new hint for this text field..
     *
     * @param hint hint to set.
     */
    public void setHint(String hint) {
        this.hint = hint;
        applyHint();
    }

    /**
     * Applies hint, if needed.
     */
    private void applyHint() {
        if (isVisible()) {
            Hinter.applyHint(this, hint);
        } else {
            Hinter.applyHint(this, null);
        }
    }

    /**
     * Invoked when component state changes.
     *
     * @param e the component event
     */
    protected final void processComponentEvent(ComponentEvent e) {
        super.processComponentEvent(e);
        applyHint();
    }
}